
import javax.media.*;
import java.awt.event.*;
import java.awt.Component;

import javax.swing.*;
import javax.swing.Timer;

import java.io.*;
import java.net.*;

class PlayerFacade
	extends WindowAdapter
	implements ControllerListener, ActionListener
{
	private Player jmfPlayer = null;
	private TimeObserver timeObserver = null;
	private Timer timer;

	protected JFrame videoFrame = null;
	private boolean isMediaLoaded= false;
	private boolean isPlaying = false;
	private boolean repeat = true;
	private boolean mediaHasVideo = false;

	public PlayerFacade()
	{
		timer = new Timer(1000, this);
		videoFrame = new JFrame();
	}

	public void setUrl(URL url)
	{
		try
		{
			if(isMediaLoaded == true) stop();

			jmfPlayer = Manager.createPlayer(url);
			jmfPlayer.addControllerListener((ControllerListener) this);
			jmfPlayer.realize();
			isMediaLoaded = true;
		}
		catch(IOException e)
		{
			//new MessageBox("JMF Error", e.toString());
		}
		catch(NoPlayerException e)
		{
			//new MessageBox("JMF Error", e.toString());
		}
		catch(NullPointerException e)
		{
			//new MessageBox("JMF Error", e.toString());
		}
	}

	/**
	 * Sets a time observer for the media player timing events
	 * @param o Object to be notified of timing events
	 */
	public void setTimeObserver(TimeObserver o)
	{
		timeObserver = o;
	}
	
	/**
	 * Disables timing notification
	 */
	public void unsetTimeObserver()
	{
		timeObserver = null;
	}
	
	/**
	 * Start media playback
	 */
	public void start()
	{
		if (jmfPlayer == null) return;

		if(isPlaying == false)
		{
			videoFrame.setVisible(mediaHasVideo);
			timer.start();
			jmfPlayer.start();
			isMediaLoaded = true;
			isPlaying = true;
		}
	}

	/**
	 * Stops media playback
	 */
	public void stop()
	{
		if (jmfPlayer == null) return;

		if (isPlaying == true)
		{
			videoFrame.setVisible(false);
			timer.stop();
			jmfPlayer.setMediaTime(new Time(0));
			jmfPlayer.stop();
			
			timeObserver.updateTime(0);
			
			jmfPlayer.deallocate();
			
			isMediaLoaded = false;
			isPlaying = false;
		}

		if(videoFrame != null)
		{
			videoFrame.setVisible(false);
		}
	}

	/**
	 * Pauses media playback. If playback is already paused, it will be resumed.
	 */
	public void pause(){

		if (jmfPlayer == null) return;
		
		if(isPlaying == true)
		{
			isPlaying = false;
			jmfPlayer.stop();
		}
		else
		{
			jmfPlayer.start();
			isPlaying = true;
		}      
	}

	/**
	 * Advances playbak by 5 seconds.
	 */
	public void forward()
	{
		if (jmfPlayer == null) return;

		Time t = new Time(0);
		t = jmfPlayer.getMediaTime();
		
		double newPosition = t.getSeconds() + 5;
		Time newTime = new Time(newPosition);
		jmfPlayer.setMediaTime(newTime);
	}

	/**
	 * Repositions playback five seconds before the current position.
	 */
	public void rewind()
	{
		if (jmfPlayer == null) return;

		Time t = new Time(0);
		t = jmfPlayer.getMediaTime();
		
		double newPosition = t.getSeconds() - 5;
		Time newTime = new Time(newPosition);
		
		jmfPlayer.setMediaTime(newTime);
	}

	/**
	 * Updates playback position. New position is given in seconds from the
	 * begining of the media file.
	 * 
	 * @param secs New position in seconds.
	 */
	public void setPosition(int secs)
	{
		if (jmfPlayer == null) return;

		Time t = new Time((double)secs);
		jmfPlayer.setMediaTime(t);
	}

	/**
	 * Sets playback mode to one shot or repeat.
	 * @param mode True for repeat playback, false for one shot.
	 */
	public void setRepeatMode(boolean mode)
	{
		repeat = mode;
	}

	/**
	 * Enables or disables sound output.
	 * @param value True disables sound.
	 */
	public void setMute(boolean value)
	{
		if (jmfPlayer != null)
		{
			GainControl gainCtl = jmfPlayer.getGainControl();
			gainCtl.setMute(value);
		}
	}

	/**
	 * Updates playback speed.
	 * @param newRate One of 0.5, 1 or 2
	 */
	public void setRate(float newRate)
	{
		if(newRate != 0.5 && newRate != 1.0 && newRate != 2.0 )
			return;

		jmfPlayer.setRate(newRate);
	}

	/**
	 * Modifies output volume.
	 * @param percentage New volume expressed in percentage. 100 is max gain.
	 */
	public void setVolume(int percentage)
	{
		GainControl gainCtl = jmfPlayer.getGainControl();
		gainCtl.setLevel( (float)percentage / (float)100 );
	}

	/**
	 * Allows calling objects to get current media position in seconds
	 */
	public Time getMediaTime()
	{
		if (jmfPlayer == null) return null;
		return jmfPlayer.getMediaTime();
	}
	
	public void controllerUpdate(ControllerEvent ce)
	{
		if (ce instanceof RealizeCompleteEvent)
		{
			Component visualComponent = jmfPlayer.getVisualComponent();
			
			if (visualComponent != null)
			{
				mediaHasVideo = true;
				videoFrame.add(visualComponent);
				videoFrame.pack();
			}
			
			jmfPlayer.prefetch();
		}
		else if (ce instanceof PrefetchCompleteEvent)
		{
			int totalSecs = (int)jmfPlayer.getDuration().getSeconds();

			if (timeObserver != null)
				timeObserver.updateTotalTime(totalSecs);

			start();
		}
		else if (ce instanceof EndOfMediaEvent)
		{
			if(repeat == false)
			{
				stop();
			}
			else
			{
				stop();
				start();
			}
		}
	}

	public void actionPerformed(ActionEvent ae)
	{
		int elapsedSecs = (int)jmfPlayer.getMediaTime().getSeconds();

		if (timeObserver != null)
			timeObserver.updateTime(elapsedSecs);
	}
}